#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  /// <summary>
  /// Represents a support vector solution for a classification problem which can be visualized in the GUI.
  /// </summary>
  [Item("SupportVectorClassificationSolution", "Represents a support vector solution for a classification problem which can be visualized in the GUI.")]
  [StorableClass]
  public sealed class SupportVectorClassificationSolution : ClassificationSolution {

    public new SupportVectorMachineModel Model {
      get { return (SupportVectorMachineModel)base.Model; }
    }

    public Dataset SupportVectors {
      get { return CalculateSupportVectors(); }
    }

    [StorableConstructor]
    private SupportVectorClassificationSolution(bool deserializing) : base(deserializing) { }
    private SupportVectorClassificationSolution(SupportVectorClassificationSolution original, Cloner cloner)
      : base(original, cloner) {
    }
    public SupportVectorClassificationSolution(SupportVectorMachineModel model, IClassificationProblemData problemData)
      : base(model, problemData) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SupportVectorClassificationSolution(this, cloner);
    }

    protected override void OnProblemDataChanged(EventArgs e) {
      Model.Model.SupportVectorIndizes = new int[0];
      base.OnProblemDataChanged(e);
    }

    private Dataset CalculateSupportVectors() {
      if (Model.Model.SupportVectorIndizes.Length == 0)
        return new Dataset(new List<string>(), new double[0, 0]);

      double[,] data = new double[Model.Model.SupportVectorIndizes.Length, ProblemData.Dataset.Columns];
      for (int i = 0; i < Model.Model.SupportVectorIndizes.Length; i++) {
        for (int column = 0; column < ProblemData.Dataset.Columns; column++)
          data[i, column] = ProblemData.Dataset[Model.Model.SupportVectorIndizes[i], column];
      }
      return new Dataset(ProblemData.Dataset.VariableNames, data);
    }
  }
}
